package be.iminds.ilabt.jfed.experiment.setup.config;

import be.iminds.ilabt.jfed.rspec.adv_based_generator.AdvertisementBasedRspecGeneratorConfig;
import com.fasterxml.jackson.annotation.*;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Collections;

@JsonInclude(JsonInclude.Include.NON_DEFAULT)
@JsonIgnoreProperties(value={"@context"})
public class GenerateUsingAdvertisementRequestRSpec extends RequestRSpec {
    @Nonnull
    private final AdvertisementBasedRspecGeneratorConfig
            config;

    public GenerateUsingAdvertisementRequestRSpec() {
        super(RequestRSpecSource.GENERATE_USING_ADVERTISEMENT, false, false);
        this.config = new AdvertisementBasedRspecGeneratorConfig(Collections.singletonList(
                new AdvertisementBasedRspecGeneratorConfig.NodeInfo(
                        1, null,"node", null,
                        null, null, null, null, null,
                        null, null, null,
                        null, null, true, null
                )
        ));
    }

    @JsonCreator
    public GenerateUsingAdvertisementRequestRSpec(
            @Nonnull @JsonProperty("source") RequestRSpecSource source,
            @Nullable @JsonProperty("autoAssignSpecificNodesUsingListResources") Boolean autoAssignSpecificNodesUsingListResources,
            @Nullable @JsonProperty("useNitosLease") Boolean useNitosLease,
            @Nullable @JsonProperty("config") AdvertisementBasedRspecGeneratorConfig config) {
        super(source, autoAssignSpecificNodesUsingListResources, useNitosLease);
        assert source == RequestRSpecSource.GENERATE_USING_ADVERTISEMENT;
        assert this.getSource() == RequestRSpecSource.GENERATE_USING_ADVERTISEMENT;
        this.config =
                config == null ?
                        new GenerateUsingAdvertisementRequestRSpec().getConfig() :
                        config;
    }

    @JsonIgnore
    public boolean isValid() {
        if (config == null)
            throw new IllegalArgumentException("Config Error: rspec.config required for rspec.source == "+source);
        if (config.getNodes().isEmpty())
            throw new IllegalArgumentException("Config Error: rspec.config.nodes may not be empty for rspec.source == "+source);
        return true;
    }

    @JsonProperty
    @Nonnull
    public AdvertisementBasedRspecGeneratorConfig getConfig() {
        return config;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof GenerateUsingAdvertisementRequestRSpec)) return false;
        if (!super.equals(o)) return false;

        GenerateUsingAdvertisementRequestRSpec that = (GenerateUsingAdvertisementRequestRSpec) o;

        //noinspection ConstantConditions
        return config != null ? config.equals(that.config) : that.config == null;
    }

    @Override
    public int hashCode() {
        int result = super.hashCode();
        //noinspection ConstantConditions
        result = 31 * result + (config != null ? config.hashCode() : 0);
        return result;
    }

    @Override
    public String toString() {
        return "GenerateUsingAdvertisementRequestRSpec{" +
                "source=" + this.getSource() +
                ", autoAssignSpecificNodesUsingListResources=" + this.getAutoAssignSpecificNodesUsingListResources() +
                ", useNitosLease=" + this.getUseNitosLease() +
                ", config=" + config +
                '}';
    }
}
